%=========================================================================%
% Mutual repressor switch
%
% Deterministic Fractional Occupancy Model
% iGEM 2012 Team Slovenia - http://2012.igem.org/Team:Slovenia
%=========================================================================%


TIME = 1200;            % Simulation duration.
SamplesPerStep = 2000;
SAMPLES = TIME*SamplesPerStep;
dt = 1/SamplesPerStep;  % Time step.


% Graphs data.
grafTAL97_KRAB = zeros(1,SAMPLES);
grafTAL57_KRAB = zeros(1,SAMPLES);
grafBFP = zeros(1,SAMPLES);
grafMCitrin = zeros(1,SAMPLES);
counter=1;     


% Initial values/concentrations.
BFP = 0;
MCitrin = 0;
TAL97_KRAB = 0;
TAL57_KRAB = 0;
PIPKRAB = 0;
EKRAB = 0;

Signal1 = 0;
Signal2 = 0;


% Fractional occupancy parameters - association constants (k1-k4) and exponents (n1-n4).
k1 = 1;     
k2 = 1;
k3 = 1;
k4 = 1;
n1 = 1.15;
n2 = 1.15;
n3 = 1.15;
n4 = 1.15;

Ka = 3;     % Amount of activator required for 50% activation (minimal promoter).
Kr = 1;     % Amount of repressor required for 50% repression (constitutive promoter).


% Production & degradation parameters.
k_BFP = 10;              % BFP production rate (active promoter).
kb_BFP = 0.00;           % BFP basal rate (leaking - inactive promoter).
deg_BFP = 0.1;           % BFP degradation rate.

k_citrin = 10;              % MCitrin production rate (active pormoter).
kb_citrin = 0.00;           % MCitrin basal rate (leaking - inactive promoter).
deg_citrin = 0.1;           % MCitrin degradation rate.

k1_TAL97_KRAB = 10;         % TAL97_KRAB production rate from construct 1.
kb1_TAL97_KRAB = 0.00;      % TAL97_KRAB basal rate from construct 1.
k3_TAL97_KRAB = 10;         % TAL97_KRAB production rate from construct 3.
kb3_TAL97_KRAB = 0.00;      % TAL97_KRAB basal rate from construct 3.
deg_TAL97_KRAB = 0.1;       % TAL97_KRAB degradation rate.

k2_TAL57_KRAB = 10;         % TAL57_KRAB production rate from construct 2.
kb2_TAL57_KRAB = 0.00;      % TAL57_KRAB basal rate from construct 2.
k4_TAL57_KRAB = 10;         % TAL57_KRAB production rate from construct 4.
kb4_TAL57_KRAB = 0.00;      % TAL57_KRAB basal rate from construct 4.
deg_TAL57_KRAB = 0.1;       % TAL57_KRAB degradation rate.

k_PIPKRAB = 10;             % PIPKRAB production rate.
deg_PIPKRAB = 0.1;          % PIPKRAB degradation rate.

k_EKRAB = 10;              % EKRAB production rate.
deg_EKRAB = 0.1;           % EKRAB degradation rate.

deg_PIPKRAB_fast = 500;    % PIPKRAB degradation rate when signal1 is present.
deg_PIPKRAB_normal = 0.1;   % PIPKRAB degradation rate when signal1 is absent (normal degradation rate).

deg_EKRAB_fast = 500;     % EKRAB degradation rate when signal2 is present.
deg_EKRAB_normal = 0.1;    % EKRAB degradation rate when signal2 is absent (normal degradation rate).


%========== SIMULATION ==========%
for t=0:dt:(TIME-1)                   
    % State 1.
    if t==0
       Signal1 = 1;       
    end
    if t==100
       Signal1 = 0;
    end

    % State 2.
    if t==400
      Signal2= 1;
    end
    if t==500
      Signal2 = 0;
    end
    
    % State 1.
    if t==800
      Signal1 = 1;
    end
    if t==900
      Signal1 = 0;
    end   


    
    % Calculate fractional occupancies
    f1 = Kr/(Kr+k1*TAL57_KRAB^n1);            % Construct 1 (BFP).
    f2 = Kr/(Kr+k2*TAL97_KRAB^n2);            % Construct 2 (citrin).    
    f3 = Kr/(Kr+k3*PIPKRAB^n3);               % Construct 3.    
    f4 = Kr/(Kr+k4*EKRAB^n4);                 % Construct 4.    
    
    % Signal1 present -> PIPKRAB cannot bind (cannot repress construct 3).
    % This is modeled as fast PIPKRAB degradation in the presence of signal1.
    if Signal1==1
        deg_PIPKRAB = deg_PIPKRAB_fast;
    else
        deg_PIPKRAB = deg_PIPKRAB_normal;
    end
    
    % Signal2 present -> EKRAB cannot bind (cannot repress construct 4).
    if Signal2==1
        deg_EKRAB = deg_EKRAB_fast;
    else
        deg_EKRAB = deg_EKRAB_normal;
    end
    
    
    % ODEs.
    % Form: dProtein/dt = production_rate * P(active promoter) + basal_rate*P(inactive promoter) - degradation_rate*Protein
    dBFP = dt * (k_BFP*f1 + kb_BFP*(1-f1) - deg_BFP*BFP);
    dMCitrin = dt * (k_citrin*f2 + kb_citrin*(1-f2) - deg_citrin*MCitrin);
    dTAL97_KRAB = dt * (k1_TAL97_KRAB*f1 + kb1_TAL97_KRAB*(1-f1) + k3_TAL97_KRAB*f3 + kb3_TAL97_KRAB*(1-f3) - deg_TAL97_KRAB*TAL97_KRAB);
    dTAL57_KRAB = dt * (k2_TAL57_KRAB*f2 + kb2_TAL57_KRAB*(1-f2) + k4_TAL57_KRAB*f4 + kb4_TAL57_KRAB*(1-f4) - deg_TAL57_KRAB*TAL57_KRAB);
    dPIPKRAB = dt * (k_PIPKRAB - deg_PIPKRAB*PIPKRAB);
    dEKRAB = dt * (k_EKRAB - deg_EKRAB*EKRAB);
    
    % Update values.
    BFP = BFP + dBFP;
    MCitrin = MCitrin + dMCitrin;
    TAL97_KRAB = TAL97_KRAB + dTAL97_KRAB;
    TAL57_KRAB = TAL57_KRAB + dTAL57_KRAB;
    PIPKRAB = PIPKRAB + dPIPKRAB;
    EKRAB = EKRAB + dEKRAB;
    
    if (PIPKRAB < 0) PIPKRAB = 0; end
    if (EKRAB < 0) EKRAB = 0; end
    
    
    % Graphs.
    grafBFP(counter) = BFP;
    grafMCitrin(counter) = MCitrin;
    counter = counter+1;   
end

%========== PLOTTING ==========%
set(0,'DefaultLineLineSmoothing','on');
set(0,'DefaultPatchLineSmoothing','on');
set(gcf,'DefaultAxesColorOrder',[0.0 0.2 0.9; 0.85 0.4 0.0;]);
set(gcf,'Color',[1 1 1]);
set(gcf,'renderer','zbuffer');
set(0,'DefaultAxesFontWeight','normal')

plot((1:SAMPLES)/SamplesPerStep,[grafBFP' grafMCitrin']);
legend('BFP', 'mCitrine');
title('Mutual repressor switch');

grid on;

max1 = max(grafBFP);
max2 = max(grafMCitrin);
max3 = 0;
axis([0 TIME-1 -0.2 max(max1,max(max2,max3))+5]);
screensize = get(0, 'ScreenSize');
set(gcf,'Position',[screensize(3)/2 - 350, screensize(4)/2 - 160, 1000, 480]);
legend('Orientation','Horizontal', 'Location','NorthOutside');
legend boxoff 


% export_fig:
% https://sites.google.com/site/oliverwoodford/software/export_fig
export_fig '.\MutualSwitch_test2b.png'


% Save figure data to text file
time = (1:SAMPLES)/SamplesPerStep;
fid = fopen('MutualSwitch_test2b_time.txt','wt');
fprintf(fid,'%f\n',time(1:100:end));
fclose(fid);

fid = fopen('MutualSwitch_test2b_BFP.txt','wt');
fprintf(fid,'%f\n',grafBFP(1:100:end)');
fclose(fid);

fid = fopen('MutualSwitch_test2b_Cit.txt','wt');
fprintf(fid,'%f\n',grafMCitrin(1:100:end)');
fclose(fid);

disp('Done.');

